c
c     ************************************************************************
c     *                                                                      *
c     *                                                                      *
c     *     *******  *******  *******  *******  *******  *******  *******    *
c     *     *     *  *     *  *        *     *  *     *  *     *  *          *
c     *     *     *  *     *  *        *     *  *     *  *     *  *          *
c     *     *******  *******  *******  *******  *******  *     *  *          * 
c     *     *        *   *    *        *        *   *    *     *  *          * 
c     *     *        *    *   *        *        *    *   *     *  *          * 
c     *     *        *     *  *******  *        *     *  *******  *******    * 
c     *                                                                      *
c     *                                                                      *
c     *                       *   * ***** *****    *****   *   ***** *****   *
c     *                       *   * *     *   *    *   *   *   *   * *       *
c     *                       *   * ***** *****    *   *   *   *   * *****   *
c     *                        * *  *     *  *  ** *   *   *   *   *     *   *
c     *                         *   ***** *   * ** *****   *   ***** *****   *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
c     ************************************************************************
c     *                                                                      *
c     *  program   :                                                         *
c     *                                                                      *
c     *               preproc_v0105.f                                        *
c     *                                                                      *
c     *  objective :                                                         *
c     *                                                                      *
c     *               to conduct pre-process for gauge observations,         *
c     *               satellite estimates and GDAS outputs                   *
c     *                                                                      *
c     *  algorithm :                                                         *
c     *                                                                      *
c     *               1)  read raw data;                                     *
c     *                                                                      *
c     *               2)  conduct quality control for the raw GTS station    *
c     *                   observations and satellite estimation data;        *
c     *                                                                      *
c     *               3)  construct gauge-based analysis using Shepard       *
c     *                   interpolation method;                              *
c     *                                                                      *
c     *               4)  write out the GDAS data, quality controled         *
c     *                   satellite estimates and gauge-based analysis       *
c     *                   based on the quality controled GTS gauge data      *
c     *                   into files on hard disk so that the merging        *
c     *                   program can use them as inputs.                    *
c     *                                                                      *
c     *  input     :                                                         *
c     *                                                                      *
c     *                                                                      *
c     *               PART I  ( land masks )                                 *
c     *                                                                      *
c     *                 unit 11  == land/ocean mask                          *
c     *                             at 0.1 deg lat/lon grid points           *
c     *                             land = 1.0; ocean = 0.0;                 *
c     *                                                                      *
c     *                 unit 12  == land portion in percentage               *
c     *                             at 0.25 deg lat/lon grid boxes           *
c     *                             all land = 100.0%; all ocean = 0.0%;     *
c     *                                                                      *
c     *                 unit 13  == GTS station list over Africa             *
c     *                                                                      *
c     *                                                                      *
c     *               PART II  ( raw data )                                  *
c     *                          at their original resolutions )             *
c     *                                                                      *
c     *                 unit 20  == raw data availability                    *
c     *                             kraw (kk),kk=1,5                         *
c     *                                                                      *
c     *                 unit 21  == raw GTS station data                     *
c     *                             over the target domain                   *
c     *                             krain0 (ngauge)                          *
c     *                                                                      *
c     *                 unit 22  == raw GPI estimates                        *
c     *                             at resolution of 0.10 deg grid points    *
c     *                             pgpi0 (nx000,ny000) = precip  (mm/day)   *
c     *                             sgpi0 (nx000,ny000) = sampling           *
c     *                                                                      *
c     *                 unit 23  == Fractional coverage of clouds/surface    *
c     *                             colder than 275K at 0.1 deg grd          *
c     *                             fc275 (nx000,ny000) = fractional coverage*
c     *                                                                      *
c     *                 unit 24  == raw SSM/I estimates                      *
c     *                             at resolution of 0.25 deg grid boxes     *
c     *                             pssm0 ( nx025, ny025 ) = precip (mm/day) *
c     *                             sssm0 ( nx025, ny025 ) = sampling        *
c     *                                                                      *
c     *                 unit 25  == raw AMSU  estimates                      *
c     *                             at resolution of 0.25 deg grid boxes     *
c     *                             pams0 ( nx025, ny025 ) = precip (mm/day) *
c     *                             sams0 ( nx025, ny025 ) = sampling        *
c     *                                                                      *
c     *                                                                      *
c     *                                                                      *
c     *               PART III  ( tables used in defining gauge analysis     *
c     *                                                                      *
c     *                 unit 31  == gauge-gauge distance in km               *
c     *                             for all combinations                     *
c     *                                                                      *
c     *                 unit 32  == grid-gauge distance in km                *
c     *                             for nsort closest gauges                 *  
c     *                             (number of gauge and distance in km)     *
c     *                                                                      *
c     *  outout    :                                                         *
c     *                                                                      *
c     *                 unit 71  == QCed GTS station data                    *
c     *                             over the target domain                   *
c     *                             krain1 (ngauge)                          *
c     *                                                                      *
c     *                 unit 72  == precipitation analysis at 0.1 deg lat/lon*
c     *                             grid points based on quality-controled   *
c     *                             GTS station observations                 *
c     *                             pgts ( nx000, ny000 ) = precip (mm/day)  *
c     *                             sgts ( nx000, ny000 ) = gauge number     *
c     *                             egts ( nx000, ny000 ) = error  (mm/day)  *
c     *                             dgts ( nx000, ny000 ) = distance (km)    *
c     *                                                                      *
c     *                 unit 73  == QCed GPI estimates                       *
c     *                             at resolution of 0.10 deg grid points    *
c     *                             pgpi1 ( nx000, ny000 ) = precip (mm/day) *
c     *                             sgpi1 ( nx000, ny000 ) = sampling        *
c     *                                                                      *
c     *                 unit 74  == QCed SSM/I Estimates                     *
c     *                             at resolution of 0.25 deg grid boxes     *
c     *                             pssm1 ( nx025, ny025 ) = precip (mm/day) *
c     *                             sssm1 ( nx025, ny025 ) = sampling        *
c     *                                                                      *
c     *                 unit 75  == QCd AMSU Estimates                       *
c     *                             at resolution of 0.50 deg grid boxes     *
c     *                             pams1 ( nx050, ny050 ) = precip (mm/day) *
c     *                             sams1 ( nx050, ny050 ) = sampling        *
c     *                                                                      *
c     *                                                                      *
c     *                                                                      *
c     *  programmer  :                                                       *
c     *                                                                      *
c     *                 Pingping Xie  / Vadlamani Kumar / Tim Love           *
c     *                 NOAA Climate Prediction Center                       *
c     *                 5200 Auth Road, #805B                                *
c     *                 Camp Springs, MD 20746                               *
c     *                                                                      *
c     *                 Tel.    (301)763-8000, ext.7572                      *
c     *                 Fax     (301)763-8125                                *
c     *                 E-mail: xping@sgi17.wwb.noaa.gov                     *
c     *                 E-mail: Tim.Love@noaa.gov                            *
c     *                                                                      *
c     *  date        :                                                       *
c     *                                                                      *
c     *                 coded    on July 1, 1999       ( V9907 )             *
c     *                 modified on September 21, 1999 ( V9909 )             *
c     *                 modified on December  20, 1999 ( V9912 )             *
c     *                 modified on 2001-05-30         ( V0105 )             *
c     *                 modified on 2005-08-22 for fews(lnx)                 *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
      include  'preproc_v0500_parameter.h' 
c
c
      dimension   kraw (5),
     #            fc275  (nx000,ny000)
      integer*2   krain0 (ngauge)  
      dimension   pgpi0  (nx000,ny000), sgpi0  (nx000,ny000),
     #            pssm0  (nx025,ny025), sssm0  (nx025,ny025),
     #            pams0  (nx025,ny025), sams0  (nx025,ny025)
c
      integer*2   krain1 (ngauge)
      dimension   pgpi1 (nx000,ny000), sgpi1 (nx000,ny000),
     #            pssm1 (nx025,ny025), sssm1 (nx025,ny025),
     #            pams1 (nx025,ny025), sams1 (nx025,ny025), 
     #            pgts1 (nx000,ny000), sgts1 (nx000,ny000), 
     #            egts1 (nx000,ny000), dgts1 (nx000,ny000)  
c  
c
c     ************************************************************************
c     *                                                                      *
c     *  1.  to open the data files                                          *
c     *                                                                      *
c     ************************************************************************
      print *, '  ' 
      print *, '*******************************************************' 
      print *, '  ' 
      print *, ' ::: STEP 2: PREPROCESS ::: ' 
      print *, '  ' 
c 
      open  (unit=11,
     #       file='../lib/mask_land_africa_000.linux.gra', 
     #       access='direct',status='old',recl=nx000*ny000*4) 
      open  (unit=12,
     #       file='../lib/mask_land_portion_africa_025.linux.gra', 
     #       access='direct',status='old',recl=nx025*ny025*4) 
      open  (unit=13,       
     #       file='../lib/africa_gts_station_list.txt',
     #       access='sequential',status='old',form='formatted') 
c
      open  (unit=20,
     #       file = 'africa_raw_status.txt',
     #       access='sequential',status='old',form='formatted')
      open  (unit=21, 
     #       file = 'africa_raw_gts_station.bin',
     #       access='direct',status='old',recl=ngauge*2)
      open  (unit=22,
     #       file = 'africa_raw_gpi.bin',
     #       access='direct',status='old',recl=nx000*ny000*4) 
      open  (unit=23, 
     #       file = 'africa_raw_af275.bin',  
     #       access='direct',status='old',recl=nx000*ny000*4) 
      open  (unit=24, 
     #       file = 'africa_raw_ssmi.bin',  
     #       access='direct',status='old',recl=nx025*ny025*4)
      open  (unit=25, 
     #       file = 'africa_raw_amsu.bin',        
     #       access='direct',status='old',recl=nx025*ny025*4)
     

c
      open  (unit=31,
     #       file = '../lib/dist_gauge_gauge_africa_all.bin',
     #       access='direct',status='old',recl=ngauge*ngauge*4)
      open  (unit=32,
     #       file = '../lib/dist_grid_gauge_africa.bin',  
     #       access='direct',status='old',recl=nsort*4)
c
      open  (unit=71,
     #       file = 'africa_qc_gts_station.bin',
     #       access='direct',status='unknown',recl=nword*4)
      open  (unit=72, 
     #       file='africa_input_gts_000.gra', 
     #       access='direct',status='unknown',recl=nx000*ny000*4)
      open  (unit=73, 
     #       file='africa_input_gpi_000.gra',             
     #       access='direct',status='unknown',recl=nx000*ny000*4)
      open  (unit=74, 
     #       file='africa_input_ssmi_025.gra',             
     #       access='direct',status='unknown',recl=nx025*ny025*4)
      open  (unit=75, 
     #       file='africa_input_amsu_025.gra',            
     #       access='direct',status='unknown',recl=nx025*ny025*4)

c
c
      write  (6,*)  'started preproc !!' 
c
c     ************************************************************************
c     *                                                                      *
c     *  2.  to read in the land mask and the individual raw data            *
c     *                                                                      *
c     ************************************************************************
c
      write  (6,*)  '     started get_raw !!'  
      call  get_raw  ( kraw,
     #                 fc275,
     #                 krain0,
     #                 pgpi0, sgpi0,  
     #                 pssm0, sssm0, 
     #                 pams0, sams0 ) 

      write  (6,*)  '     finished get_raw !!'  
c
c     ************************************************************************
c     *                                                                      *
c     *  3.  to conduct quality control                                      *
c     *                                                                      *
c     ************************************************************************
c
      write  (6,*)  '     started get_qua !!'
      call  get_qua  ( kraw,
     #                 fc275, 
     #                 krain0,
     #                 pgpi0, sgpi0,  
     #                 pssm0, sssm0, 
     #                 pams0, sams0, 
     #                 krain1,
     #                 pgpi1, sgpi1, 
     #                 pssm1, sssm1, 
     #                 pams1, sams1  )  
      write  (6,*)  '     finished get_qua !!'
c
c     ************************************************************************
c     *                                                                      *
c     *  4.  to construct gauge-based analysis                               *
c     *                                                                      *
c     ************************************************************************
c
      write  (6,*)  '     started get_ana !!'
      if  (kraw(1).eq.1)  then 
        call   get_ana  ( krain1,
     #                    pgts1, sgts1, egts1, dgts1 )
      else 
        do 4001 jj = 1, ny000 
        do 4001 ii = 1, nx000 
          pgts1 (ii,jj) = undef 
          sgts1 (ii,jj) = undef 
          egts1 (ii,jj) = undef 
          dgts1 (ii,jj) = undef 
 4001   continue 
      end  if      
      write  (6,*)  '     finished get_ana !!'
      write  (6,*)  (kraw(kk),kk=1,5) 
c
c     ************************************************************************
c     *                                                                      *
c     *  5.  to conduct the scaling for satellite estimates                  *
c     *                                                                      *
c     ************************************************************************
c
      write  (6,*)  '     started get_scl !!'
      if  (kraw(1).eq.1)  then 
        call  get_scl  (  kraw,
     #                    pgts1, sgts1,   
     #                    pgpi1, pssm1, pams1 )
      end  if  
      write  (6,*)  '     finished get_scl !!' 
c
c     ************************************************************************
c     *                                                                      *
c     *  6.  to output results                                               *
c     *                                                                      *
c     ************************************************************************
c
      write  (6,*)  '     started get_out !!'
      call   get_out  ( krain1,
     #                  pgts1, sgts1, egts1, dgts1, 
     #                  pgpi1, sgpi1,
     #                  pssm1, sssm1,
     #                  pams1, sams1 )

      write  (6,*)  '     finished get_out !!'
c
      write  (6,*)  'finished preproc !!' 
c
      stop 
      end 
c
c
c
c     ************************************************************************
c     *                                                                      *
c     *                                                                      *
c     *     *******  *******  *******           *******     *     *     *    *
c     *     *        *           *              *     *    * *    *     *    *
c     *     *        *           *              *     *   *   *   *  *  *    *
c     *     *  ****  *******     *              *******  *******  *  *  *    * 
c     *     *  *  *  *           *              *   *    *     *  * * * *    * 
c     *     *     *  *           *              *    *   *     *  **   **    * 
c     *     *******  *******     *     *******  *     *  *     *  *     *    * 
c     *                                                                      *
c     *                                                                      *
c     *                                                                      *
c     *                       *   * ***** *****    ***** ***** ***** *****   *
c     *                       *   * *     *   *    *   * *   * *   * *   *   *
c     *                       *   * ***** *****    ***** ***** *   * *****   *
c     *                        * *  *     *  *  **     *     * *   *     *   *
c     *                         *   ***** *   * ** ***** ***** ***** *****   *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
c     ************************************************************************
c     *                                                                      *
c     *   subroutine:                                                        *
c     *                                                                      *
c     *                get_raw                                               *
c     *                                                                      *
c     *   objective :                                                        *
c     *                                                                      *
c     *                to get the raw data for construction of precipitation *
c     *                analysis over Africa                                  *
c     *                                                                      *
c     *   input     :                                                        *
c     *                                                                      *
c     *                                                                      *
c     *               PART I  ( land masks )                                 *
c     *                                                                      *
c     *                 unit 11  == land/ocean mask                          *
c     *                             at 0.1 deg lat/lon grid points           *
c     *                             land = 1.0; ocean = 0.0;                 *
c     *                                                                      *
c     *                 unit 12  == land portion in percentage               *
c     *                             at 0.25 deg lat/lon grid boxes           *
c     *                             all land = 100.0%; all ocean = 0.0%;     *
c     *                                                                      *
c     *                 unit 13  == GTS station list over Africa             *
c     *                                                                      *
c     *                                                                      *
c     *               PART II  ( raw data )                                  *
c     *                          at their original resolutions )             *
c     *                                                                      *
c     *                 unit 20  == raw data availability                    *
c     *                                                                      *
c     *                                                                      *
c     *                 unit 21  == raw GTS station data                     *
c     *                             over the target domain                   *
c     *                             krain0 (ngauge)                          *
c     *                                                                      *
c     *                 unit 22  == raw GPI estimates                        *
c     *                             at resolution of 0.10 deg grid points    *
c     *                             pgpi0 (nx000,ny000) = precip  (mm/day)   *
c     *                             sgpi0 (nx000,ny000) = sampling           *
c     *                                                                      *
c     *                 unit 23  == Fractional coverage of clouds/surface    *
c     *                             colder than 275K at 0.1 deg grd          *
c     *                             fc275 (nx000,ny000) = fractional coverage*
c     *                                                                      *
c     *                 unit 24  == raw SSM/I estimates                      *
c     *                             at resolution of 0.25 deg grid boxes     *
c     *                             pssm0 ( nx025, ny025 ) = precip (mm/day) *
c     *                             sssm0 ( nx025, ny025 ) = sampling        *
c     *                                                                      *
c     *                 unit 25  == raw AMSU  estimates                      *
c     *                             at resolution of 0.50 deg grid boxes     *
c     *                             pams0 ( nx050, ny050 ) = precip (mm/day) *
c     *                             sams0 ( nx050, ny050 ) = sampling        *
c     *                                                                      *
c     *                                                                      *
c     *                                                                      *
c     *  outout    :                                                         *
c     *                                                                      *
c     *                 fc275    ==  fractional coverage at 275K at 0.1 deg  *
c     *                                                                      *
c     *                 krain0   ==  raw GTS station observations            *
c     *                                                                      *
c     *                 pgpi0    ==  raw GPI Precip (mm/day) at 0.1 deg Grd  *
c     *                                                                      *
c     *                 sgpi0    ==  raw GPI Sampling at 0.1 deg Grd         *
c     *                                                                      *
c     *                 pssm0    ==  raw SSM/I precip (mm/day) at 0.25 deg   *
c     *                                                                      *
c     *                 sssm0    ==  raw SSM/I sampling at 0.25 deg Grd      *
c     *                                                                      *
c     *                 pams0    ==  raw AMSU Precip (mm/day) at 0.5 deg Grd *
c     *                                                                      *
c     *                 sams0    ==  raw AMSU Sampling at 0.5 deg Grd        *
c     *                                                                      *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
      subroutine    get_raw  ( kraw,
     #                         fc275,
     #                         krain0,
     #                         pgpi0, sgpi0,      
     #                         pssm0, sssm0, 
     #                         pams0, sams0 )

c 
      include  'preproc_v0500_parameter.h' 
c
      dimension   kraw (5),
     #            fc275  (nx000,ny000)
      integer*2   krain0 (ngauge)
      dimension   pgpi0  (nx000,ny000), sgpi0  (nx000,ny000),
     #            pssm0  (nx025,ny025), sssm0  (nx025,ny025),
     #            pams0  (nx025,ny025), sams0  (nx025,ny025)
c
c
c     ************************************************************************
c     *                                                                      *
c     *  1.  to get the land/ocean masks                                     *
c     *                                                                      *
c     ************************************************************************
c
      read  (11,rec=1)   rland000 
      read  (12,rec=1)   rland025 
      write  (6,*)   '          finished reading land masks !!' 
c
c     ************************************************************************
c     *                                                                      *
c     *  2.  to get the GTS station list                                     *
c     *                                                                      *
c     ************************************************************************
c
      do 2001 kk = 1, ngauge 
        read  (13,2901)  rlat (kk), rlon (kk) 
 2001 continue 
 2901 format  (20x,2f10.2) 
      write  (6,*)   '          finished reading GTS station list !!' 
c
c     ************************************************************************
c     *                                                                      *
c     *  3.  to read raw data availability                                   *
c     *                                                                      *
c     ************************************************************************
c
      read  (20,*)  (kraw(kk),kk=1,5) 
      write  (6,*)   '          finished reading raw data status !!' 
c
c     ************************************************************************
c     *                                                                      *
c     *  4.  to read the GTS station observations of precipitation           *
c     *                                                                      *
c     ************************************************************************
c
      read  (21,rec=1)   krain0  
c        print *, "get_raw:",krain0
      write  (6,*)   '          finished reading raw data for GTS !!' 
c
c     ************************************************************************
c     *                                                                      *
c     *  5.  to read the GPI defined by Al Herman                            *
c     *                                                                      *
c     ************************************************************************
c
      read  (22,rec=1)   pgpi0  
      read  (22,rec=2)   sgpi0  
      write  (6,*)   '          finished reading raw data for GPI !!' 
c
c     ************************************************************************
c     *                                                                      *
c     *  6.  to read the fractional coverage at 275K                         *
c     *                                                                      *
c     ************************************************************************
c
      read  (23,rec=1)   fc275 
      write  (6,*)   '          finished reading raw data for FC275 !!' 
c
c     ************************************************************************
c     *                                                                      *
c     *  7.  to read the SSM/I Estimates                                     *
c     *                                                                      *
c     ************************************************************************
c
      read  (24,rec=1)   pssm0 
      read  (24,rec=2)   sssm0  
      write  (6,*)   '          finished reading raw data for SSM/I !!' 
c
c
c     ************************************************************************
c     *                                                                      *
c     *  8.  to read the AMSU Estimates                                      *
c     *                                                                      *
c     ************************************************************************
c
      read  (25,rec=1)   pams0  
      read  (25,rec=2)   sams0  
      write  (6,*)   '          finished reading raw data for AMSU !!' 
c

c
c     ************************************************************************
c     *                                                                      *
c     *  10.  to close the raw data files                                    *
c     *                                                                      *
c     ************************************************************************
c
      close  (unit=11) 
      close  (unit=12) 
      close  (unit=13) 
      close  (unit=20) 
      close  (unit=21) 
      close  (unit=22) 
      close  (unit=23) 
      close  (unit=24) 
      close  (unit=25) 
      close  (unit=26) 
c
      return 
      end 
c
c
c
c     ************************************************************************
c     *                                                                      *
c     *                                                                      *
c     *     *******  *******  *******           *******  *     *     *       *
c     *     *        *           *              *     *  *     *    * *      *
c     *     *        *           *              *     *  *     *   *   *     *
c     *     *  ****  *******     *              *     *  *     *  *******    * 
c     *     *  *  *  *           *              *   * *  *     *  *     *    * 
c     *     *     *  *           *              *    **  *     *  *     *    * 
c     *     *******  *******     *     *******  *******  *******  *     *    * 
c     *                                                                      *
c     *                                                                      *
c     *                                                                      *
c     *                       *   * ***** *****    ***** ***** ***** *****   *
c     *                       *   * *     *   *    *   * *   * *   *     *   *
c     *                       *   * ***** *****    ***** ***** *   *     *   *
c     *                        * *  *     *  *  **     *     * *   *     *   *
c     *                         *   ***** *   * ** ***** ***** *****     *   *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
c     ************************************************************************
c     *                                                                      *
c     *   subroutine:                                                        *
c     *                                                                      *
c     *                get_qua                                               *
c     *                                                                      *
c     *   objective :                                                        *
c     *                                                                      *
c     *                to conduct quality control for the raw data           *
c     *                                                                      *
c     *   input     :                                                        *
c     *                                                                      *
c     *                kraw    ==  raw data availability                     *
c     *                                                                      *
c     *                                                                      *
c     *                fc275   ==  fractional coverage at 275K               *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                krain0  ==  raw GTS station observation data          *
c     *                            ngauge integers in total                  *
c     *                                                                      *
c     *                pgpi0   ==  raw GPI precip                            *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                sgpi0   ==  raw GPI sampling                          *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                pssm0   ==  raw SSM/I precip                          *
c     *                            at 0.25 deg grid boxes                    *
c     *                                                                      *
c     *                sssm0   ==  raw SSM/I sampling                        *
c     *                            at 0.25 deg grid boxes                    *
c     *                                                                      *
c     *                pams0   ==  raw AMSU precp                            *
c     *                            at 0.50 deg grid boxes                    *
c     *                                                                      *
c     *                sams0   ==  raw AMSU sampling                         *
c     *                            at 0.50 deg grid boxes                    *
c     *                                                                      *
c     *   output    :                                                        *
c     *                                                                      *
c     *                                                                      *
c     *                krain1  ==  QCed GTS station observation data         *
c     *                            ngauge integers in total                  *
c     *                                                                      *
c     *                pgpi1   ==  QCed GPI precip                           *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                sgpi1   ==  QCed GPI sampling                         *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                pssm1   ==  QCed SSM/I precip                         *
c     *                            at 0.25 deg grid boxes                    *
c     *                                                                      *
c     *                sssm1   ==  QCed SSM/I sampling                       *
c     *                            at 0.25 deg grid boxes                    *
c     *                                                                      *
c     *                pams1   ==  QCed AMSU precp                           *
c     *                            at 0.50 deg grid boxes                    *
c     *                                                                      *
c     *                sams1   ==  QCed AMSU sampling                        *
c     *                            at 0.50 deg grid boxes                    *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
      subroutine   get_qua  ( kraw,
     #                        fc275, 
     #                        krain0,
     #                        pgpi0, sgpi0,  
     #                        pssm0, sssm0, 
     #                        pams0, sams0, 
     #                        krain1,
     #                        pgpi1, sgpi1, 
     #                        pssm1, sssm1, 
     #                        pams1, sams1  )  
c 
      include  'preproc_v0500_parameter.h' 
c
      dimension   kraw (5),
     #            fc275  (nx000,ny000) 
      integer*2   krain0 (ngauge)   
      dimension   pgpi0  (nx000,ny000), sgpi0  (nx000,ny000), 
     #            pssm0  (nx025,ny025), sssm0  (nx025,ny025), 
     #            pams0  (nx025,ny025), sams0  (nx025,ny025) 
c
      integer*2   krain1 (ngauge)                  
      dimension   pgpi1 (nx000,ny000), sgpi1 (nx000,ny000),
     #            pssm1 (nx025,ny025), sssm1 (nx025,ny025),
     #            pams1 (nx025,ny025), sams1 (nx025,ny025)
c
      dimension   fc025 (nx025,ny025), fc050 (nx050,ny050), 
     #            pssm050 (nx050,ny050), 
     #            rland050 (nx050,ny050)    
c   
c
c     ************************************************************************
c     *                                                                      *
c     *  1.  to conduct the quality control for the GPI estimates            *
c     *                                                                      *
c     ************************************************************************
c
      call  qua_gpi ( kraw,
     #                nx000, ny000, 
     #                pgpi0, sgpi0, pgpi1, sgpi1, 
     #                undef                      ) 
      write  (6,*)   '          finished QC for GPI !!'
c
c     ************************************************************************
c     *                                                                      *
c     *  2.  to conduct the quality control for the SSM/I estimates          *
c     *                                                                      *
c     ************************************************************************
c
      call   qua_ssmi  ( kraw,
     #                   nx000, ny000, fc275,
     #                   nx025, ny025, fc025, rland025,
     #                   pssm0, sssm0, pssm1, sssm1,
     #                   undef                       )
      write  (6,*)   '          finished QC for SSM/I !!' 
c
c     ************************************************************************
c     *                                                                      *
c     *  3.  to conduct the quality control for the AMSU estimates           *
c     *                                                                      *
c     ************************************************************************
c
      call   qua_amsu  ( kraw,
     #                   nx000, ny000, fc275,
     #                   nx025, ny025, fc025, rland025,
     #                   pams0, sams0, pams1, sams1,
     #                   undef                       )
      write  (6,*)   '          finished QC for AMSU !!'       
c
c
c     ************************************************************************
c     *                                                                      *
c     *  4.  to conduct the quality control for the GTS station observations *
c     *                                                                      *
c     ************************************************************************
c
       call  qua_gts  ( kraw,
     #                  nx000,  ny000,  fc275, pgpi1,
     #                          nx025,  ny025, pssm1, 
     #                          nx050,  ny050, pams1, 
     #                          ngauge, rlat,  rlon,    
     #                          krain0,   krain1,
     #                          undef                       )

      write  (6,*)   '          finished QC for GTS !!'        
c
c
      return 
      end  
c
c
c
c     ************************************************************************
c     *                                                                      *
c     *            *****  *   *    *           *****  *****  *****           *
c     *            *   *  *   *   * *          *      *   *    *             *
c     *            * * *  *   *  *****         * ***  *****    *             *
c     *            *  **  *   *  *   *         *   *  *        *             *
c     *            *****  *****  *   *  *****  *****  *      *****           *
c     *                                                                      *
c     ************************************************************************
c
c     ************************************************************************
c     *                                                                      *
c     *   subroutine:                                                        *
c     *                                                                      *
c     *                qua_gpi                                               *
c     *                                                                      *
c     *   objective :                                                        *
c     *                                                                      *
c     *                to conduct quality control for the GPI estimates      *
c     *                                                                      *
c     *   arguments :                                                        *
c     *                                                                      *
c     *                kraw  =  raw data availability                        *
c     *                                                                      *
c     *                nx    =  x-dimension for the GPI                      *
c     *                                                                      *
c     *                ny    =  y-dimension for the GPI                      *
c     *                                                                      *
c     *                pgpi0 = raw GPI precip                                *
c     *                                                                      *
c     *                sgpi0 = raw GPI sampling                              *
c     *                                                                      *
c     *                pgpi1 = QCed GPI precip                               *
c     *                                                                      *
c     *                sgpi1 = QCed GPI sampling                             *
c     *                                                                      *
c     *                undef =  undefined code                               *
c     *                                                                      *
c     *   algorithm  :                                                       *
c     *                                                                      *
c     *                No quality control is conducted for GPI               *
c     *                in this version.                                      *
c     *                                                                      *
c     ************************************************************************
c   
      subroutine   qua_gpi  ( kraw,
     #                        nx, ny, 
     #                        pgpi0, sgpi0, 
     #                        pgpi1, sgpi1, 
     #                        undef )   
c
      dimension   kraw (5),
     #            pgpi0 (nx,ny), sgpi0 (nx,ny), 
     #            pgpi1 (nx,ny), sgpi1 (nx,ny) 
c
c
c     ************************************************************************
c     *                                                                      *
c     *  1.  to conduct quality control for the GPI                          *
c     *                                                                      *
c     ************************************************************************
c
      do 1001 jj = 1, ny 
      do 1001 ii = 1, nx 
        pgpi1 (ii,jj) = pgpi0 (ii,jj) 
        sgpi1 (ii,jj) = sgpi0 (ii,jj) 
 1001 continue 
c
      return 
      end 
c
c
c
c     ************************************************************************
c     *                                                                      *
c     *        *****  *   *    *           *****  *****  *   *  *****        *
c     *        *   *  *   *   * *          *      *      ** **    *          *
c     *        * * *  *   *  *****         *****  *****  * * *    *          *
c     *        *  **  *   *  *   *             *      *  *   *    *          *
c     *        *****  *****  *   *  *****  *****  *****  *   *  *****        *
c     *                                                                      *
c     ************************************************************************
c
c     ************************************************************************
c     *                                                                      *
c     *   subroutine:                                                        *
c     *                                                                      *
c     *                qua_ssmi                                              *
c     *                                                                      *
c     *   objective :                                                        *
c     *                                                                      *
c     *                to conduct quality control for the SSM/I estimates    *
c     *                                                                      *
c     *   arguments :                                                        *
c     *                                                                      *
c     *                kraw     =  raw data availability                     *
c     *                                                                      *
c     *                nx000    =  x-dimension in 0.10 grid points           *
c     *                                                                      *
c     *                ny000    =  y-dimension in 0.10 grid points           *
c     *                                                                      *
c     *                fc000    =  FC275 at 0.10 grid points                 *
c     *                                                                      *
c     *                nx025    =  x-dimension in 0.25 deg grid boxes        *
c     *                                                                      *
c     *                ny025    =  y-dimension in 0.25 deg grid boxes        *
c     *                                                                      *
c     *                fc025    =  FC275 at 0.25 grid boxes                  *
c     *                                                                      *
c     *                rland025 =  percentage of land in a grid box          *
c     *                                                                      *
c     *                rain0    =  raw SSM/I estimates                       *
c     *                                                                      *
c     *                samp0    =  raw SSM/I sampling                        *
c     *                                                                      *
c     *                rain1    =  QCed SSM/I estimates                      *
c     *                                                                      *
c     *                samp1    =  QCed SSM/I sampling                       *
c     *                                                                      *
c     *                undef =  undefined code                               *
c     *                                                                      *
c     *   algorithm  :                                                       *
c     *                                                                      *
c     *                1)  convert fc275 at 0.1 deg grid box into 0.25 deg   *
c     *                                                                      *
c     *                2)  set rain = 0.0 if fc275 is 0.0 (no clouds)        *
c     *                                                                      *
c     *                3)  set rain = 0.0 if fc275<0.1 but rain>0.2 mm/day   *
c     *                                                                      *
c     *                4)  set rain = 0.0 if the target grid is over the     *
c     *                    coast and there is only upto 1 neighboring grid   *
c     *                    with raining                                      *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c   
      subroutine   qua_ssmi  ( kraw,
     #                         nx000, ny000, fc000, 
     #                         nx025, ny025, fc025, rland025,  
     #                         rain0, samp0, rain1, samp1, 
     #                         undef                       )  
c
      dimension     kraw  (5),
     #              fc000 (nx000,ny000), 
     #              fc025 (nx025,ny025), rland025 (nx025,ny025),  
     #              rain0 (nx025,ny025), samp0 (nx025,ny025), 
     #              rain1 (nx025,ny025), samp1 (nx025,ny025) 
c
c
c     ************************************************************************
c     *                                                                      *
c     *  1.  to set the rainfall less than 0.1 mm/day as 0.0                 *
c     *                                                                      *
c     ************************************************************************
c
      do 1001 jj = 1, ny025
      do 1001 ii = 1, nx025
        rain1 (ii,jj) = rain0 (ii,jj)
        samp1 (ii,jj) = samp0 (ii,jj)
        if  (rain0(ii,jj).ne.undef.and.
     #       rain0(ii,jj).lt.0.1)  then
          rain1 (ii,jj) = 0.0
        end  if
 1001 continue
c
c     ************************************************************************
c     *                                                                      *
c     *  2.  to regrid the FC275 from 0.1 deg to 0.25 deg grid box           *
c     *                                                                      *
c     ************************************************************************
c
      if  (kraw(3).ne.1)  then 
        go  to  9999 
      end  if  
c
      do 2002 jj = 1, ny025     
      do 2002 ii = 1, nx025     
        mj = mod (jj,2) 
        nj = jj / 2 
        if  (mj.eq.1)  then 
          j1 = nj * 5 + 1  
        else 
          j1 = (nj-1) * 5 + 3  
        end  if   
        j2 = j1 + 3 
        mi = mod (ii,2)       
        ni = ii / 2 
        if  (mi.eq.1)  then 
          i1 = ni * 5 + 1   
        else 
          i1 = (ni-1) * 5 + 3
        end  if   
        i2 = i1 + 3
        nn = 0
        ss = 0.0
        do 2001 j = j1, j2
        do 2001 i = i1, i2
          if  (fc000(i,j).ne.undef)  then 
            nn = nn + 1
            ss = ss + fc000 (i,j)
          end  if
 2001   continue
        if  (nn.gt.0)  then
          fc025 (ii,jj) = ss / nn  
        else
          fc025 (ii,jj) = undef    
        end  if
 2002 continue
c
c     ************************************************************************
c     *                                                                      *
c     *  3.  to set the no-cloud grid box as non-rain                        *
c     *                                                                      *
c     ************************************************************************
c
      do 3001 jj = 1, ny025   
      do 3001 ii = 1, nx025   
        if  (rain1(ii,jj).gt.0.0)  then 
          if  (fc025(ii,jj).eq.0.0)  then 
            rain1 (ii,jj) = 0.0 
          else  
     #      if  (fc025(ii,jj).le.0.1.and.
     #           rain1(ii,jj).gt.5.0)  then 
            rain1 (ii,jj) = 0.0 
          end  if  
        end  if  
 3001 continue  
c
c     ************************************************************************
c     *                                                                      *
c     *  4.  to set rain as 0.0 based on neighboring grid values             *
c     *                                                                      *
c     ************************************************************************
c
      kdg = 1 
      do 4003 jj = 1, ny025   
      do 4003 ii = 1, nx025  
        if  (rain1(ii,jj).gt.0.0)  then 
          j1 = jj - kdg  
          if  (j1.lt.1)  then 
            j1 =  1
          end  if  
          j2 = jj + kdg  
          if  (j2.gt.ny025)  then 
            j2 = ny025   
          end if  
          i1 = ii - kdg  
          if  (i1.lt.1)  then 
            i1 = 1 
          end  if  
          i2 = ii + kdg  
          if  (i2.gt.nx025)  then 
            i2 = nx025  
          end  if  
          nn = 0 
          ss = 0.0 
          sf = 0.0 
          do 4001 j = j1, j2 
          do 4001 i = i1, i2 
            nn = nn + 1 
            ss = ss + rland025 (i,j) 
            sf = sf + fc025    (i,j) 
 4001     continue 
          ss = ss / nn 
          sf = sf / nn 
          if  (ss.gt.0.0.and.
     #         ss.lt.100.0.and.
     #         fc025(ii,jj).le.0.1)  then 
            nr = 0 
            do 4002 j = j1, j2 
            do 4002 i = i1, i2 
              if  (rain1(i,j).gt.0.0)  then 
                nr = nr + 1 
              end  if  
 4002       continue 
            if  (nr.le.2)  then 
              rain1 (ii,jj) = 0.0 
            end  if  
          end  if  
        end  if  
 4003 continue  
c
c
 9999 return  
      end 
c
c
c
c     ************************************************************************
c     *                                                                      *
c     *        *****  *   *    *             *    *   *  *****  *   *        *
c     *        *   *  *   *   * *           * *   ** **  *      *   *        *
c     *        * * *  *   *  *****         *****  * * *  *****  *   *        *
c     *        *  **  *   *  *   *         *   *  *   *      *  *   *        *
c     *        *****  *****  *   *  *****  *   *  *   *  *****  *****        *
c     *                                                                      *
c     ************************************************************************
c     ************************************************************************
c     *                                                                      *
c     *   subroutine:                                                        *
c     *                                                                      *
c     *                qua_amsu                                              *
c     *                                                                      *
c     *   objective :                                                        *
c     *                                                                      *
c     *                to conduct quality control for the AMSUB estimates    *
c     *                                                                      *
c     *   arguments :                                                        *
c     *                                                                      *
c     *                kraw     =  raw data availability                     *
c     *                                                                      *
c     *                nx000    =  x-dimension in 0.10 grid points           *
c     *                                                                      *
c     *                ny000    =  y-dimension in 0.10 grid points           *
c     *                                                                      *
c     *                fc000    =  FC275 at 0.10 grid points                 *
c     *                                                                      *
c     *                nx025    =  x-dimension in 0.25 deg grid boxes        *
c     *                                                                      *
c     *                ny025    =  y-dimension in 0.25 deg grid boxes        *
c     *                                                                      *
c     *                fc025    =  FC275 at 0.25 grid boxes                  *
c     *                                                                      *
c     *                rland025 =  percentage of land in a grid box          *
c     *                                                                      *
c     *                rain0    =  raw AMSUB estimates                       *
c     *                                                                      *
c     *                samp0    =  raw AMSUB sampling                        *
c     *                                                                      *
c     *                rain1    =  QCed AMSUB estimates                      *
c     *                                                                      *
c     *                samp1    =  QCed AMSUB sampling                       *
c     *                                                                      *
c     *                undef =  undefined code                               *
c     *                                                                      *
c     *   algorithm  :                                                       *
c     *                                                                      *
c     *                1)  convert fc275 at 0.1 deg grid box into 0.25 deg   *
c     *                                                                      *
c     *                2)  set rain = 0.0 if fc275 is 0.0 (no clouds)        *
c     *                                                                      *
c     *                3)  set rain = 0.0 if fc275<0.1 but rain>0.2 mm/day   *
c     *                                                                      *
c     *                4)  set rain = 0.0 if the target grid is over the     *
c     *                    coast and there is only upto 1 neighboring grid   *
c     *                    with raining                                      *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c   
      subroutine   qua_amsu  ( kraw,
     #                         nx000, ny000, fc000, 
     #                         nx025, ny025, fc025, rland025,  
     #                         rain0, samp0, rain1, samp1, 
     #                         undef                       )  
c
      dimension     kraw  (5),
     #              fc000 (nx000,ny000), 
     #              fc025 (nx025,ny025), rland025 (nx025,ny025),  
     #              rain0 (nx025,ny025), samp0 (nx025,ny025), 
     #              rain1 (nx025,ny025), samp1 (nx025,ny025) 
c
c
c     ************************************************************************
c     *                                                                      *
c     *  1.  to set the rainfall less than 0.1 mm/day as 0.0                 *
c     *                                                                      *
c     ************************************************************************
c
      do 1001 jj = 1, ny025
      do 1001 ii = 1, nx025
        rain1 (ii,jj) = rain0 (ii,jj)
        samp1 (ii,jj) = samp0 (ii,jj)
        if  (rain0(ii,jj).ne.undef.and.
     #       rain0(ii,jj).lt.0.1)  then
          rain1 (ii,jj) = 0.0
        end  if
 1001 continue
c
c     ************************************************************************
c     *                                                                      *
c     *  2.  to regrid the FC275 from 0.1 deg to 0.25 deg grid box           *
c     *                                                                      *
c     ************************************************************************
c
      if  (kraw(3).ne.1)  then 
        go  to  9999 
      end  if  
c
      do 2002 jj = 1, ny025     
      do 2002 ii = 1, nx025     
        mj = mod (jj,2) 
        nj = jj / 2 
        if  (mj.eq.1)  then 
          j1 = nj * 5 + 1  
        else 
          j1 = (nj-1) * 5 + 3  
        end  if   
        j2 = j1 + 3 
        mi = mod (ii,2)       
        ni = ii / 2 
        if  (mi.eq.1)  then 
          i1 = ni * 5 + 1   
        else 
          i1 = (ni-1) * 5 + 3
        end  if   
        i2 = i1 + 3
        nn = 0
        ss = 0.0
        do 2001 j = j1, j2
        do 2001 i = i1, i2
          if  (fc000(i,j).ne.undef)  then 
            nn = nn + 1
            ss = ss + fc000 (i,j)
          end  if
 2001   continue
        if  (nn.gt.0)  then
          fc025 (ii,jj) = ss / nn  
        else
          fc025 (ii,jj) = undef    
        end  if
 2002 continue
c
c     ************************************************************************
c     *                                                                      *
c     *  3.  to set the no-cloud grid box as non-rain                        *
c     *                                                                      *
c     ************************************************************************
c
      do 3001 jj = 1, ny025   
      do 3001 ii = 1, nx025   
        if  (rain1(ii,jj).gt.0.0)  then 
          if  (fc025(ii,jj).eq.0.0)  then 
            rain1 (ii,jj) = 0.0 
          else  
     #      if  (fc025(ii,jj).le.0.1.and.
     #           rain1(ii,jj).gt.5.0)  then 
            rain1 (ii,jj) = 0.0 
          end  if  
        end  if  
 3001 continue  
c
c     ************************************************************************
c     *                                                                      *
c     *  4.  to set rain as 0.0 based on neighboring grid values             *
c     *                                                                      *
c     ************************************************************************
c
      kdg = 1 
      do 4003 jj = 1, ny025   
      do 4003 ii = 1, nx025  
        if  (rain1(ii,jj).gt.0.0)  then 
          j1 = jj - kdg  
          if  (j1.lt.1)  then 
            j1 =  1
          end  if  
          j2 = jj + kdg  
          if  (j2.gt.ny025)  then 
            j2 = ny025   
          end if  
          i1 = ii - kdg  
          if  (i1.lt.1)  then 
            i1 = 1 
          end  if  
          i2 = ii + kdg  
          if  (i2.gt.nx025)  then 
            i2 = nx025  
          end  if  
          nn = 0 
          ss = 0.0 
          sf = 0.0 
          do 4001 j = j1, j2 
          do 4001 i = i1, i2 
            nn = nn + 1 
            ss = ss + rland025 (i,j) 
            sf = sf + fc025    (i,j) 
 4001     continue 
          ss = ss / nn 
          sf = sf / nn 
          if  (ss.gt.0.0.and.
     #         ss.lt.100.0.and.
     #         fc025(ii,jj).le.0.1)  then 
            nr = 0 
            do 4002 j = j1, j2 
            do 4002 i = i1, i2 
              if  (rain1(i,j).gt.0.0)  then 
                nr = nr + 1 
              end  if  
 4002       continue 
            if  (nr.le.2)  then 
              rain1 (ii,jj) = 0.0 
            end  if  
          end  if  
        end  if  
 4003 continue  
c
c
 9999 return  
      end 
c
c
c     ************************************************************************
c     *                                                                      *
c     *            *****  *   *    *           *****  *****  *****           *
c     *            *   *  *   *   * *          *        *    *               *
c     *            * * *  *   *  *****         * ***    *    *****           *
c     *            *  **  *   *  *   *         *   *    *        *           *
c     *            *****  *****  *   *  *****  *****    *    *****           *
c     *                                                                      *
c     ************************************************************************
c
c     ************************************************************************
c     *                                                                      *
c     *   subroutine:                                                        *
c     *                                                                      *
c     *                qua_gts                                               *
c     *                                                                      *
c     *   objective :                                                        *
c     *                                                                      *
c     *                to conduct quality control for the GTS station data   *
c     *                                                                      *
c     *   arguments :                                                        *
c     *                                                                      *
c     *                kraw    =  raw data availability                      *
c     *                                                                      *
c     *                nx000   =  x-dimension in 0.10 grid system            *
c     *                                                                      *
c     *                ny000   =  y-dimension in 0.10 grid system            *
c     *                                                                      *
c     *                fc000   =  FC275 at 0.1 grid system                   *
c     *                                                                      *
c     *                pgpi000 =  GPI rain at 0.1 grid system                *
c     *                                                                      *
c     *                nx025   =  x-dimension in 0.25 grid system            *
c     *                                                                      *
c     *                ny025   =  y-dimension in 0.25 grid system            *
c     *                                                                      *
c     *                pssm025 =  SSM/I rain at 0.25 grid system             *
c     *                                                                      *
c     *                nx050   =  x-dimension in 0.50 grid system            *
c     *                                                                      *
c     *                ny050   =  y-dimension in 0.50 grid system            *
c     *                                                                      *
c     *                pams050 =  AMSU rain at 0.50 grid system              *
c     *                                                                      *
c     *                ngauge  =  number of gauges available in Africa       *
c     *                                                                      *
c     *                rlat    =  latitude of gauges                         *
c     *                                                                      *
c     *                rlon    =  longitude of gauges                        *
c     *                                                                      *
c     *                krain0  =  raw GTS gauge report in 0.1 mm/day         *
c     *                                                                      *
c     *                krain1  =  QCed GTS gauge report in 0.1 mm/day        *
c     *                                                                      *
c     *                undef =  undefined code                               *
c     *                                                                      *
c     *   algorithm  :                                                       *
c     *                                                                      *
c     *                set the GTS as missing under the following situations *
c     *                                                                      *
c     *                1)  GTS >  0.0  &  FC275 = 0.0                        *
c     *                                                                      *
c     *                2)  GTS <  0.1  &  ALL SATE > 2.0                     *
c     *                                                                      *
c     *                3)  GTS > 50.0  &  ALL SATE < 20.0                    *
c     *                                                                      *
c     *                4)  GTS <  5.0  &  ALL SATE > 20.0 & ALL DIFF > 20.0  *
c     *                                                                      *
c     *                5)  GTS > 20.0  &  ALL SATE < 1.0                     *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
       subroutine   qua_gts  ( kraw,
     #                         nx000,  ny000, fc000,   pgpi000,
     #                         nx025,  ny025, pssm025,  
     #                         nx050,  ny050, pams025,  
     #                         ngauge, rlat,  rlon,    
     #                         krain0,   krain1,  
     #                         undef                       )
c
      dimension   kraw  (5),
     #            fc000   (nx000,ny000), pgpi000 (nx000,ny000),
     #            pssm025 (nx025,ny025), 
     #            pams025 (nx025,ny025), 
     #            rlat (ngauge), rlon (ngauge) 
c
       integer*2  krain0 (ngauge), krain1 (ngauge) 
c
c     ************************************************************************
c     *                                                                      *
c     *  1.  to initialize the quality controled gauge data                  *
c     *      Only reads if gtts precip < 300.0 mm per day.                   *
c     *                                                                      *
c     ************************************************************************
c
      kundef = -999 
      do 1001 kk = 1, ngauge 
        if  (krain0 (kk).le.3000)  then 
          krain1 (kk) = krain0 (kk) 
        else 
          krain1 (kk) = kundef 
        end  if  
 1001 continue 
c
c
c     ************************************************************************
c     *                                                                      *
c     *  2.  to check the raw gauge data by comparison with satellite data   *
c     *                                                                      *
c     ************************************************************************
c
      kdg000 = 2 
      kdg025 = 1 
      kdg050 = 1 
c
      nn = 0 
      do 2005 kk = 1, ngauge 
        if  (krain1(kk).ne.kundef)  then 
c
c         to determine the closest satellite pixels at various resolutions 
          ic000 = nint ((rlon(kk)+20.000)/0.10) + 1
          jc000 = nint ((rlat(kk)+40.000)/0.10) + 1
          ic025 = nint ((rlon(kk)+19.875)/0.25) + 1 
          jc025 = nint ((rlat(kk)+39.875)/0.25) + 1 
          ic050 = nint ((rlon(kk)+19.750)/0.50) + 1 
          jc050 = nint ((rlat(kk)+39.750)/0.50) + 1
          if  (ic000.ge.1.and.ic000.le.nx000.and. 
     #         jc000.ge.1.and.jc000.le.ny000.and. 
     #         ic025.ge.1.and.ic025.le.nx025.and. 
     #         jc025.ge.1.and.jc025.le.ny025.and. 
     #         ic050.ge.1.and.ic050.le.nx050.and. 
     #         jc050.ge.1.and.jc050.le.ny050)  then 
c
c            to determine the comparison domain 
             js000 = jc000 - kdg000 
             if  (js000.lt.1)  then 
               js000 = 1 
             end  if  
             jn000 = jc000 + kdg000 
             if  (jn000.gt.ny000)  then 
               jn000 = ny000 
             end  if  
             iw000 = ic000 - kdg000 
             if  (iw000.lt.1)  then 
               iw000 = 1 
             end  if  
             ie000 = ic000 + kdg000 
             if  (ie000.gt.nx000)  then 
               ie000 = nx000 
             end  if  
             js025 = jc025 - kdg025 
             if  (js025.lt.1)  then 
               js025 = 1 
             end  if  
             jn025 = jc025 + kdg025  
             if  (jn025.gt.ny025)  then 
               jn025 = ny025  
             end  if  
             iw025 = ic025 - kdg025 
             if  (iw025.lt.1)  then 
               iw025 = 1 
             end  if  
             ie025 = ic025 + kdg025 
             if  (ie025.gt.nx025)  then 
               ie025 = nx025 
             end  if
             js050 = jc050 - kdg050  
             if  (js050.lt.1)  then 
               js050 = 1 
             end  if  
             jn050 = jc050 + kdg050  
             if  (jn050.gt.ny050)  then 
               jn050 = ny050    
             end  if  
             iw050 = ic050 - kdg050  
             if  (iw050.lt.1)  then 
               iw050 = 1 
             end  if  
             ie050 = ic050 + kdg050  
             if  (ie050.gt.nx050)  then 
               ie050 = nx050  
             end  if
c
             nn = nn + 1 
c                  
c            to compare with FC275 
             nf = 0 
             sf = 0.0 
             do 2001 jj = js000, jn000 
             do 2001 ii = iw000, ie000 
               if  (fc000(ii,jj).ge.0.0)  then  
                 nf = nf + 1 
                 sf = sf + fc000 (ii,jj) 
               end  if  
 2001        continue 
             if  (nf.gt.0)  then 
               sf = sf / nf 
               if  (sf.eq.0.0.and. 
     #              krain1(kk).gt.10)  then
      print *, '------ >>> QC #1, thrown out', krain1(kk) 
                 krain1 (kk) = kundef 
                 go  to  2005 
               end  if  
             end  if  
c
c            to calculate mean GPI rain  
             n1 = 0 
             s1 = 0.0 
             do 2002 jj = js000, jn000
             do 2002 ii = iw000, ie000
               if  (pgpi000(ii,jj).ge.0.0)  then
                 n1 = n1 + 1
                 s1 = s1 + pgpi000 (ii,jj)
               end  if
 2002        continue 
             if  (n1.gt.0)  then 
               s1 = s1 / n1 
             else 
               s1 = undef 
             end  if  
c
c            to calculate mean SSM/I rain
             n2 = 0
             s2 = 0.0
             do 2003 jj = js025, jn025 
             do 2003 ii = iw025, ie025 
               if  (pssm025(ii,jj).ge.0.0)  then
                 n2 = n2 + 1
                 s2 = s2 + pssm025 (ii,jj)
               end  if
 2003        continue
             if  (n2.gt.0)  then 
               s2 = s2 / n2  
             else 
               s2 = undef 
             end  if  
c
c            to calculate mean AMSU rain
             n3 = 0
             s3 = 0.0
             do 2004 jj = js025, jn025  
             do 2004 ii = iw025, ie025  
               if  (pams025(ii,jj).ge.0.0)  then
                 n3 = n3 + 1
                 s3 = s3 + pams025 (ii,jj)
               end  if
 2004        continue
             if  (n3.gt.0)  then 
               s3 = s3 / n3  
             else 
               s3 = undef 
             end  if  
c
c            to check on no-rain reports (gauge<0.1mm/day)  
             if  (krain1(kk).lt.1.and. 
     #            s1        .gt.2.0.and. 
     #            s2        .gt.2.0.and. 
     #            s3        .gt.2.0    )  then
      print *, '------ >>> QC #2, thrown out', krain1(kk) 
               krain1 (kk) = kundef 
               go  to  2005 
             end  if  
c
c            to check on heavy-rain reports (gauge>50.0mm/day) 
             if  (krain1(kk).gt.500.and.
     #            s1        .ne.undef.and.
     #            s1        .lt.20.0.and.
     #            s2        .ne.undef.and.
     #            s2        .lt.20.0.and.
     #            s3        .ne.undef.and.
     #            s3        .lt.20.0    )  then
      print *, '------ >>> QC #3, thrown out', krain1(kk)     
               krain1 (kk) = kundef 
               go  to  2005 
             end  if
c
c            to check on other suspicious rain reports (gauge<5.0mm/day) 
             if  (krain1(kk).lt.50.and. 
     #            s1        .ge.20.0.and.
     #            s2        .ge.20.0.and.
     #            s3        .ge.20.0    )  then
               rn = krain1 (kk) / 10.0 
               d1 = s1 - rn 
               d2 = s2 - rn 
               d3 = s3 - rn  
               if  (d1.gt.20.0.and. 
     #              d2.gt.20.0.and. 
     #              d3.gt.20.0)  then
      print *, '------ >>> QC #4, thrown out', krain1(kk) 
                 krain1 (kk) = kundef
                 go  to  2005
               end  if
             end  if  
c
c            to check on other suspicious rain reports (sate<1.0mm/day)
             if  (s1        .ne.undef.and.
     #            s1        .le.1.0.and.
     #            s2        .ne.undef.and.
     #            s2        .le.1.0.and.
     #            s3        .ne.undef.and.
     #            s3        .le.1.0.and.
     #            krain1(kk).gt.200      )  then
      print *, '------ >>> QC #5, thrown out', krain1(kk)     
               krain1 (kk) = kundef
               go  to  2005
             end  if
c
          end  if  
        end  if  
 2005 continue 
c
c        print *, "get_qua:",krain1
      return  
      end 
c
c
c
c     ************************************************************************
c     *                                                                      *
c     *                                                                      *
c     *     *******  *******  *******              *     *     *     *       *
c     *     *        *           *                * *    **    *    * *      *
c     *     *        *           *               *   *   * *   *   *   *     *
c     *     *  ****  *******     *              *******  *  *  *  *******    * 
c     *     *  *  *  *           *              *     *  *   * *  *     *    * 
c     *     *     *  *           *              *     *  *    **  *     *    * 
c     *     *******  *******     *     *******  *     *  *     *  *     *    * 
c     *                                                                      *
c     *                                                                      *
c     *                                                                      *
c     *                       *   * ***** *****    ***** ***** ***** *****   *
c     *                       *   * *     *   *    *   * *   * *   * *   *   *
c     *                       *   * ***** *****    ***** ***** *   * *****   *
c     *                        * *  *     *  *  **     *     * *   *     *   *
c     *                         *   ***** *   * ** ***** ***** ***** *****   *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
c     ************************************************************************
c     *                                                                      *
c     *   subroutine:                                                        *
c     *                                                                      *
c     *                get_ana                                               *
c     *                                                                      *
c     *   objective :                                                        *
c     *                                                                      *
c     *                to construct the gauge-based analysis using           *
c     *                the Shepard method                                    *
c     *                                                                      *
c     *   input     :                                                        *
c     *                                                                      *
c     *                krain1  ==  QCed GTS station observation data         *
c     *                            ngauge integers in total                  *
c     *                                                                      *
c     *                unit 31 == gauge-gauge distance for all combinations  *
c     *                           (km)                                       *
c     *                                                                      *
c     *                unit 32 == grid-gauge distance (nsort closest gauges) *
c     *                           (number of gauge and distance in km)       *
c     *                                                                      *
c     *   output    :                                                        *
c     *                                                                      *
c     *                                                                      *
c     *                pgts1   ==  gauge-based analysis (mm/day)             *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                sgts1   ==  number of gauges                          *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                egts1   ==  estimated gauge analysis error (mm/day)   *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                dgts1   ==  distance to the closest gauge (km)        *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
      subroutine   get_ana  ( krain1, 
     #                        pgts1, sgts1, egts1, dgts1 )   
c 
      include  'preproc_v0500_parameter.h' 
c
      integer*2   krain1 (ngauge) 
      dimension   pgts1 (nx000,ny000), sgts1 (nx000,ny000), 
     #            egts1 (nx000,ny000), dgts1 (nx000,ny000)   
c
c     ************************************************************************
c     *                                                                      *
c     *  1.  to define the gauge number for each grid                        *
c     *                                                                      *
c     ************************************************************************
c

c       print *, "get_ana:",krain1

      do 1001 jj = 1, ny000 
      do 1001 ii = 1, nx000 
        sgts1 (ii,jj) = 0.0 
 1001 continue 
      do 1002 kk = 1, ngauge 
        if  (krain1(kk).ge.0)  then 
          ii = nint ((rlon(kk) + 20.0)/0.10) + 1 
          jj = nint ((rlat(kk) + 40.0)/0.10) + 1 
          if  (ii.ge.1.and.ii.le.nx000.and.
     #         jj.ge.1.and.jj.le.ny000)  then 
            sgts1 (ii,jj) = sgts1 (ii,jj) + 1.0 
          end  if  
        end  if  
 1002 continue   

      write  (6,*)  '          finished defining gauge numbers !!' 
c
c     ************************************************************************
c     *                                                                      *
c     *  2.  to construct the gauge-based analysis                           *
c     *                                                                      *
c     ************************************************************************
c
      call  shepard  ( krain1, 
     #                 pgts1, egts1, dgts1 ) 
      write  (6,*)  '          finished calling Shepard !!' 
c
c
c     ************************************************************************
c     *                                                                      *
c     *  3.  to close the files                                              *
c     *                                                                      *
c     ************************************************************************
c
      close  (unit=31) 
      close  (unit=32) 
c
c
      return 
      end  
c
c
c
c     ************************************************************************
c     *                                                                      *
c     *                                                                      *
c     *     *******  *     *  *******  *******     *     *******  *******    *
c     *     *        *     *  *        *     *    * *    *     *   *    *    *
c     *     *        *     *  *        *     *   *   *   *     *   *    *    *
c     *     *******  *******  *******  *******  *******  *******   *    *    *
c     *           *  *     *  *        *        *     *  *   *     *    *    *
c     *           *  *     *  *        *        *     *  *    *    *    *    *
c     *     *******  *     *  *******  *        *     *  *     *  *******    *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
c     ************************************************************************
c     *                                                                      *
c     *   program   :                                                        *
c     *                shepard                                               *
c     *                                                                      *
c     *   objective :                                                        *
c     *                to interpolate the gauge precipitation using          *
c     *                the shepard algorithm                                 *
c     *                                                                      *
c     *   input     :                                                        *
c     *                                                                      *
c     *                                                                      *
c     *                krain   ==  QCed GTS station observation data         *
c     *                            ngauge integers in total                  *
c     *                                                                      *
c     *                unit 31 == gauge-gauge distance for all combinations  *
c     *                           (km)                                       *
c     *                                                                      *
c     *                unit 32 == grid-gauge distance (nsort closest gauges) *
c     *                           (number of gauge and distance in km)       *
c     *                                                                      *
c     *   output     :                                                       *
c     *                                                                      *
c     *                anal    =  analysed values at grid points             *
c     *                                                                      *
c     *                error   =  estimated interpolation error (mm)         *
c     *                                                                      *
c     *                disout  =  distance of the closest gauge              *
c     *                                                                      *
c     *   notice     :                                                       *
c     *                                                                      *
c     *                1)  you need to calculate the distances among all     *
c     *                    gauge stations and store them into a file         *
c     *                    (unit=31) before you run this program;            *
c     *                                                                      *
c     *                2)  you need to calculate the distances between       *
c     *                    each grid point and all gauges and sort out the   *
c     *                    closest ones before you run this program;         *
c     *                                                                      *
c     *                3)  please contact the programmer for more details    *
c     *                                                                      *
c     *   programmer :                                                       *
c     *                                                                      *
c     *                Pingping Xie                                          *
c     *                Climate Prediction Center                             *
c     *                5200 Auth Road, #805B                                 *
c     *                Camp Springs, MD 20746                                *
c     *                                                                      *
c     *                Tel.    (301)763-8000, ext.7572                       *
c     *                E-mail: xping@sgi17.wwb.noaa.gov                      *
c     *                                                                      *
c     ************************************************************************
c
      subroutine  shepard  ( krain, 
     #                       anal, error, disout ) 
c
      include  'preproc_v0500_parameter.h' 
c
      parameter   ( ndat  =   11 ) 
      parameter   ( re     = 6371.004, pai = 3.14159265358 )
c
      integer*2   krain(ngauge) 
      dimension   anal   (nx000,ny000), error (nx000,ny000), 
     #            disout (nx000,ny000)  
c  
      dimension   ksta (nsort), dist (nsort), 
     #            dist0(ngauge,ngauge),rain(ngauge),
     #            rinp (ndat), dist1(ndat), dist2(ndat,ndat) 
c
      dimension   kinp (ndat) 
c
c
c     1.  to read the gauge-gauge distance table 
c
      read  (31,rec=1)   dist0   
c
c     2.  to define the GTS stations observations in mm/day unit 
c
      nreport = 0 
      do 2001 kk = 1, ngauge   
        if  (krain(kk).ge.0)  then 
          nreport   = nreport + 1  
          rain (kk) = float (krain(kk)) / 10.0 
        else 
          rain (kk) = undef  
        end  if  
c          print *, "rain kk is: ",rain(kk),kk
 2001 continue 
c
c     3.  to determine the initial search radius 
c
      area = 0.0 
      ry   = re * pai / 1800.0 
      do 3002 jj = 1, ny000  
        rrlat = (jj-1)*0.1 - 40.0 
        rrlat = rrlat * pai / 180.0 
        rx    = ry * cos (rrlat) 
        a0    = rx*ry  
        do 3001 ii = 1, nx000  
          if  (rland000(ii,jj).ne.0.0)  then 
            area = area + a0 
          end  if  
 3001   continue 
 3002 continue 
      rs0 = sqrt (7.0*area/nreport/pai) 
c
c     4.  to conduct the interpolation   
c
      do 4999 jgrid = 1, ny000   
      do 4999 igrid = 1, nx000   
c         
c       4.1  to initilize the analysis
        anal  (igrid, jgrid) = undef  
        error (igrid, jgrid) = undef  
        disout(igrid, jgrid) = undef  
c
        if  (rland000(igrid,jgrid).ne.0.0)  then  
c 
c         4.2.1  to read the grid-gauge distances 
          kgrid = (jgrid-1)*nx000 + igrid   
          kinp1 = (kgrid-1)*2 + 1 
          kinp2 = kinp1 + 1 
          read  (32,rec=kinp1)   ksta 
          read  (32,rec=kinp2)   dist  
c
c         4.2.2  to select nearby stations 
          do 4001 kk = 1, ndat 
            kinp (kk) = -999.0 
            rinp (kk) = 0.0 
 4001     continue 
          ng = 0  
          ni = 0 
          do 4002 kk = 1, nsort 
            kc = ksta (kk)  
            if  (rain(kc).ge.0.0)  then 
              ng = ng + 1 
              kinp (ng) = kc  
              rinp (ng) = rain (kc) 
              dist1(ng) = dist (kk) 
              if (dist1(ng).le.rs0)  then 
                ni = ng   
              end  if  
              if  (ng.ge.ndat)  then 
                go  to  4003 
              end  if  
            end  if  
 4002     continue 
 4003     continue   
          do 4004 ll = 1, ng  
          do 4004 kk = 1, ng  
            dist2 (kk,ll) = 0.0 
 4004     continue 
          do 4006 ll = 1, ng  
            k1 = kinp (ll) 
            do 4005 kk = ll+1, ng       
              k2 = kinp (kk)  
              dist2 (kk,ll) = dist0 (k1,k2) 
              dist2 (ll,kk) = dist2 (kk,ll) 
 4005       continue  
 4006     continue 
c
c         4.2.3  to determine search radius 
          if  (ni.lt.4)  then 
            if  (ng.eq.0)  then 
              go  to  4999  
            else  if  (ng.lt.5)  then 
              rs      = dist1 (ng) + 10.0  
              nuse    = ng    
            else 
              rs      = dist1 (5)  
              nuse    = 4 
            end  if  
          else  if  (ni.ge.4.and.ni.le.10)  then 
            rs      = rs0 
            nuse    = ni 
          else 
            rs      = dist1 (11) 
            nuse    = 10 
          end  if   
c
c         4.2.5.  to calculate the interpolation 
c
          if  (nuse.ge.4)  then 
            call         interpo ( rs   ,  nuse,   ndat,
     #                             rinp ,  dist1,  dist2,
     #                             rout ,  eout             ) 
c
c             print *, rout
            anal  (igrid, jgrid) = rout 
            error (igrid, jgrid) = eout 
          else 
            anal  (igrid, jgrid) = undef 
            error (igrid, jgrid) = undef  
          end  if  
          if  (nuse.ge.1)  then 
            disout (igrid,jgrid) = dist1 (1) 
          else 
            disout (igrid,jgrid) = undef 
          end  if  
c
        end  if  
c
 4999 continue 
c
c     5.  to fill in the missing grid points 
c
      kdgrid = 5
 5000 nmiss1 = 0
      nmiss2 = 0
      do 5003 jgrid = 1, ny000
      do 5003 igrid = 1, nx000
        if  (rland000(igrid,jgrid).ne.0.0)  then
          if  (anal(igrid,jgrid).eq.undef)  then
            i0 = igrid - kdgrid
            if  (i0.lt.1)  then
              i0 = 1
            end  if
            i1 = igrid + kdgrid
            if  (i1.gt.nx000)  then
              i1 = nx000
            end  if
            j0 = jgrid - kdgrid
            if  (j0.lt.1)  then
              j0 = 1
            end  if
            j1 = jgrid + kdgrid
            if  (j1.gt.ny000)  then
              j1 = ny000
            end  if
            nn = 0
            sw = 0.0
            ss = 0.0
            do 5001 j = j0, j1
            do 5001 i = i0, i1
              if  (anal(i,j).ne.undef)  then
                w  = (i-igrid)*(i-igrid) + (j-jgrid)*(j-jgrid) + 1.0
                w  = 1.0 / w
                nn = nn + 1
                sw = sw + w
                ss = ss + w * anal (i,j)
              end  if
 5001       continue
            if  (nn.ge.5)  then
              anal (igrid,jgrid) = ss / sw
            else
              nmiss1 = nmiss1 + 1
            end  if
          end  if
          if  (error(igrid,jgrid).eq.undef)  then
            i0 = igrid - kdgrid
            if  (i0.lt.1)  then
              i0 = 1
            end  if
            i1 = igrid + kdgrid
            if  (i1.gt.nx000)  then
              i1 = nx000
            end  if
            j0 = jgrid - kdgrid
            if  (j0.lt.1)  then
              j0 = 1
            end  if
            j1 = jgrid + kdgrid
            if  (j1.gt.ny000)  then
              j1 = ny000
            end  if
            nn = 0
            sw = 0.0
            ss = 0.0
            do 5002 j = j0, j1
            do 5002 i = i0, i1
              if  (error(i,j).ne.undef)  then
                w  = (i-igrid)*(i-igrid) + (j-jgrid)*(j-jgrid) + 1.0
                w  = 1.0 / w
                nn = nn + 1
                sw = sw + w
                ss = ss + w * error (i,j)
              end  if
 5002       continue
            if  (nn.ge.5)  then
              error (igrid,jgrid) = ss / sw
            else
              nmiss2 = nmiss2 + 1
            end  if
          end  if
        end  if
 5003 continue
      if  ((nmiss1+nmiss2).gt.0)  then
        kdgrid = kdgrid + 5
        go  to  5000
      end  if
c
c
      return  
      end 
c
c
c     ************************************************************************
c     *                                                                      *
c     *                                                                      *
c     *     *******  *     *  *******  *******  *******  *******  *******    *
c     *        *     **    *     *     *        *     *  *     *  *     *    *
c     *        *     * *   *     *     *        *     *  *     *  *     *    *
c     *        *     *  *  *     *     *******  *******  *******  *     *    *
c     *        *     *   * *     *     *        *   *    *        *     *    *
c     *        *     *    **     *     *        *    *   *        *     *    *
c     *     *******  *     *     *     *******  *     *  *        *******    *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
c     ************************************************************************
c     *                                                                      *
c     *  subroutine   :                                                      *
c     *                                                                      *
c     *                   interpo ( rs   ,  ninp,  ndat,                     * 
c     *                             rinp ,  dist1, dist2,                    * 
c     *                             rout,   eout1         )                  *
c     *                                                                      *
c     *  objective    :                                                      *
c     *                                                                      *
c     *                   to interpolate the irregularly distributed         *
c     *                   observations onto grid points                      *
c     *                                                                      *
c     *  input        :                                                      *
c     *                                                                      *
c     *                   rs          =  initial search radius               *
c     *                                                                      *
c     *                                                                      *
c     *                   ninp        =  number of input observations        *
c     *                                                                      *
c     *                                                                      *
c     *                   ndat        =  size of data array                  *
c     *                                                                      *
c     *                                                                      *
c     *                   rinp        =  observation values                  *
c     *                                                                      *
c     *                                                                      *
c     *                   dist1       =  distances between the target grid   *
c     *                                  and observation stations (km)       *
c     *                                                                      *
c     *                   dist2       =  distances between all pairs of      *
c     *                                  observation stations (km)           *
c     *                                                                      *
c     *  output       :                                                      *
c     *                                                                      *
c     *                   rout        =  interpolated value at the target    *
c     *                                  grid point                          *
c     *                                                                      *
c     *                   eout        =  rout interpolation error            *
c     *                                                                      *
c     *                                                                      *
c     *  notice       :                                                      *
c     *                                                                      *
c     *                   1)  all input data are assumed to have values      *
c     *                                                                      *
c     *                   2)  the maximum station numbers (ninp) are 100     *
c     *                                                                      *
c     *                   3)  the input data should be arranged in order     *
c     *                       of increasing distance to the target grid      *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
      subroutine   interpo ( rs   ,  ninp ,  ndat,  
     #                       rinp ,  dist1,  dist2,            
     #                       rout ,  eout           )                      
c
      dimension    rinp (ndat), dist1 (ndat), dist2 (ndat,ndat) 
c
      dimension    w1 (100), w2 (100), tk (100) 
c
      doubleprecision   ang1 (100), ang2 (100,100), 
     #                  cskl, csjk, csjl, snjk, snjl, 
     #                  dcs  (100,100)          
c
      parameter  ( undef  = -999.0 ) 
      parameter  ( re     = 6371.004, pai = 3.14159265358 ) 
c
c     1.  to initilize the analysis and error estimation 
c
      rout  = undef  
      eout  = undef  
c
      if  (ninp.lt.1.or.
     #     rs.le.0.0)  then 
        go  to  9999 
      end  if  
c
c     2.  to calculate the distance-based weight  
c
      do 2001 kk = 1, ninp 
        w1 (kk) = 0.0 
        w2 (kk) = 0.0 
 2001 continue 
      if  (dist1(1).eq.0.0)  then 
        w1 (1) = 1.0 
        w2 (1) = 1.0 
        go  to  4000 
      else 
        r3 = rs / 3.0 
        do 2002 kk = 1, ninp 
          if  (dist1(kk).le.r3)  then 
            w1 (kk) = 1.0 / dist1 (kk) 
          else  if  (dist1(kk).le.rs)  then 
            c       = dist1 (kk) / rs - 1.0 
            w1 (kk) = 6.75 * c * c / rs   
          end  if  
 2002   continue 
      end  if  
      sw1 = 0.0 
      do 2003 kk = 1, ninp 
        sw1 = sw1 + w1 (kk) 
 2003 continue 
      if  (sw1.le.0.0)  then 
        go  to  9999 
      end  if  
c
c     3.  to calculate the directional adjustment factor 
c
      do 3001 kk = 1, ninp 
        ang1 (kk) = dist1 (kk) / re   
 3001 continue 
      do 3002 k2 = 1, ninp 
      do 3002 k1 = 1, ninp 
        ang2 (k1,k2) = dist2 (k1,k2) / re 
 3002 continue  
c
      do 3004 k2 = 1, ninp 
      do 3004 k1 = 1, ninp 
        dcs (k1,k2) = 1.0 
 3004 continue 
      do 3005 k2 = 1, ninp 
      do 3005 k1 = k2+1, ninp 
        cskl        = dcos (ang2(k1,k2)) 
        csjk        = dcos (ang1(k1)) 
        csjl        = dcos (ang1(k2)) 
        snjk        = dsin (ang1(k1)) 
        snjl        = dsin (ang1(k2))   
        dcs (k1,k2) = ( cskl - csjk * csjl ) / ( snjk * snjl )  
        dcs (k2,k1) = dcs (k1,k2) 
 3005 continue 
c
      do 3007 k2 = 1, ninp 
        tk (k2) = 0.0 
        do 3006 k1 = 1, ninp 
          tk (k2) = tk (k2) + w1 (k1) * (1.0 - dcs (k1,k2)) 
 3006   continue 
        tk (k2) = tk (k2) / sw1 
        w2 (k2) = w1 (k2) * w1 (k2) * ( 1.0 + tk (k2) ) 
 3007 continue 
c
c     4.  to define the interplated values at the grid point 
c
 4000 sw = 0.0 
      sr = 0.0 
      do 4001 kk = 1, ninp 
        sw = sw + w2 (kk) 
        sr = sr + w2 (kk) * rinp (kk) 
 4001 continue 
      if  (sw.gt.0.0)  then 
        rout = sr / sw 
      else 
        rout = -999.0 
      end  if  
c
c     5.  to modify the interpolated value for rain/norain 
c
      if  (ninp.ge.2)  then 
        if  (rout.le.0.2.and.
     #       rinp(1).eq.0.0.and.
     #       rinp(2).eq.0.0)  then 
          rout = 0.0 
        end  if  
      end  if  
c
c     6.  to define the error estimation 
c
      if  (ninp.ge.2)  then 
        ndt = 0 
        ave = 0.0 
        dev = 0.0 
        do 6001 kk = 1, ninp 
          ndt = ndt + 1 
          ave = ave + rinp (kk) 
          dev = dev + rinp (kk) * rinp (kk) 
 6001   continue 
        ave = ave / ndt 
        dev = dev - ndt * ave * ave 
        dev = sqrt ( dev / ndt ) 
        eout = 0.95 * dev 
      else 
        eout = -999.0 
      end  if  
c
 9999 return 
      end 
c
c
c
c     ************************************************************************
c     *                                                                      *
c     *                                                                      *
c     *     *******  *******  *******           *******  *******  *          *
c     *     *        *           *              *        *        *          *
c     *     *        *           *              *        *        *          *
c     *     *  ****  *******     *              *******  *        *          *
c     *     *  *  *  *           *                    *  *        *          *
c     *     *     *  *           *                    *  *        *          *
c     *     *******  *******     *     *******  *******  *******  *******    *
c     *                                                                      *
c     *                                                                      *
c     *                                                                      *
c     *                       *   * ***** *****    ***** ***** ***** *****   *
c     *                       *   * *     *   *    *   * *   * *   * *   *   *
c     *                       *   * ***** *****    ***** ***** *   * *****   *
c     *                        * *  *     *  *  **     *     * *   *     *   *
c     *                         *   ***** *   * ** ***** ***** ***** *****   *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
c     ************************************************************************
c     *                                                                      *
c     *   subroutine:                                                        *
c     *                                                                      *
c     *                get_scl                                               *
c     *                                                                      *  
c     *                                                                      *
c     *   objective :                                                        *
c     *                                                                      *
c     *               to scale the satellite estimates based on GTS data     *
c     *                                                                      *
c     *   input/output     :                                                 *
c     *                                                                      *
c     *                                                                      *
c     *                krain1  ==  QCed GTS station observation data         *
c     *                            ngauge integers in total                  *
c     *                                                                      *
c     *                pgts1   ==  gauge-based analysis (mm/day)             *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                pgpi1   ==  QCed GPI precip                           *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                pssm1   ==  QCed SSM/I precip                         *
c     *                            at 0.25 deg grid boxes                    *
c     *                                                                      *
c     *                pams1   ==  QCed AMSU precp                           *
c     *                            at 0.50 deg grid boxes                    *
c     *                                                                      *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
      subroutine   get_scl  ( kraw, 
     #                        pgts1, sgts1,   
     #                        pgpi1, pssm1, pams1) 
c 
      include  'preproc_v0500_parameter.h' 
c
      dimension   kraw  (5),  
     #            pgts1 (nx000,ny000), sgts1 (nx000,ny000),
     #            pgpi1 (nx000,ny000), 
     #            pssm1 (nx025,ny025),
     #            pams1 (nx050,ny050) 
c
      dimension   rain0 (nx050,ny050), samp0 (nx050,ny050), 
     #            rain1 (nx050,ny050)
c
c
c     ************************************************************************
c     *                                                                      *
c     *  1.  to define the gts analysis on 0.5 deg grid                      *
c     *                                                                      *
c     ************************************************************************
c
      do 1002 jj = 1, ny050 
      do 1002 ii = 1, nx050 
        j1 = (jj-1)*5 + 1 
        j2 = j1 + 5 
        i1 = (ii-1)*5 + 1 
        i2 = i1 + 5  
        nn = 0.0 
        s1 = 0.0 
        s2 = 0.0 
        do 1001 j = j1, j2 
        do 1001 i = i1, i2 
          if  (pgts1(i,j).ge.0.0.and. 
     #         sgts1(i,j).ge.0.0)  then 
            nn = nn + 1 
            s1 = s1 + pgts1 (i,j) 
            s2 = s2 + sgts1 (i,j) 
          end  if  
 1001   continue 
        if  (nn.gt.0)  then 
          rain0 (ii,jj) = s1 / nn 
          samp0 (ii,jj) = s2  
          nt = nt + 1 
        else 
          rain0 (ii,jj) = -999.0 
          samp0 (ii,jj) = 0.0 
        end  if  
 1002 continue   
 1999 write  (6,*)  '          finished regriding GTS !!' 
c
c     ************************************************************************
c     *                                                                      *
c     *  2.  to scale for gpi                                                *
c     *                                                                      *
c     ************************************************************************
c
      fct = -999.0 
      if  (kraw(2).ne.1)  then 
        go  to  2999  
      end  if  
c
      nt = 0 
      do 2002 jj = 1, ny050 
      do 2002 ii = 1, nx050 
        j1 = (jj-1)*5 + 1 
        j2 = j1 + 5 
        i1 = (ii-1)*5 + 1 
        i2 = i1 + 5  
        nn = 0.0 
        s1 = 0.0 
        do 2001 j = j1, j2 
        do 2001 i = i1, i2 
          if  (pgpi1(i,j).ge.0.0)  then  
            nn = nn + 1 
            s1 = s1 + pgpi1 (i,j) 
          end  if  
 2001   continue 
        if  (nn.gt.0)  then 
          rain1 (ii,jj) = s1 / nn 
          nt = nt + 1 
        else 
          rain1 (ii,jj) = -999.0 
        end  if  
 2002 continue 
c
      nn = 0 
      s0 = 0.0 
      s1 = 0.0 
      do 2003 jj = 1, ny050 
      do 2003 ii = 1, nx050 
        if  (rain0(ii,jj).ge.0.0.and. 
     #       samp0(ii,jj).ge.1.0.and. 
     #       rain1(ii,jj).ge.0.0)  then 
          nn = nn + 1 
          s0 = s0 + rain0 (ii,jj) 
          s1 = s1 + rain1 (ii,jj) 
        end  if  
 2003 continue 
c
      if  (nn.lt.20)  then 
        go  to  2999 
      else 
        if  (s1.eq.0.0)  then 
          fct = 1.0 
        else 
          fct = s0 / s1 
          if  (fct.gt.5.0)  then 
            fct = 5.0 
          end  if  
        end  if  
      end  if  
c
      fct = 1.0 
c
      do 2004 jj = 1, ny000 
      do 2004 ii = 1, nx000 
        if  (pgpi1(ii,jj).gt.0.0)  then 
          pgpi1 (ii,jj) = fct * pgpi1 (ii,jj) 
        end  if  
 2004 continue 
c
 2999 write  (6,*)  '          finished scaling for GPI !!', fct  
c
c     ************************************************************************
c     *                                                                      *
c     *  3.  to scale for ssmi                                               *
c     *                                                                      *
c     ************************************************************************
c
      fct = -999.0 
      if  (kraw(4).ne.1)  then
        go  to  3999
      end  if
c
      do 3002 jj = 1, ny050
      do 3002 ii = 1, nx050
        j1 = (jj-1)*2 + 1
        j2 = j1 + 1 
        i1 = (ii-1)*2 + 1
        i2 = i1 + 1  
        nn = 0.0
        s1 = 0.0
        do 3001 j = j1, j2
        do 3001 i = i1, i2
          if  (pssm1(i,j).ge.0.0)  then
            nn = nn + 1
            s1 = s1 + pssm1 (i,j)
          end  if
 3001   continue
        if  (nn.gt.0)  then
          rain1 (ii,jj) = s1 / nn
        else
          rain1 (ii,jj) = -999.0
        end  if
 3002 continue
c
      nn = 0
      s0 = 0.0
      s1 = 0.0
      do 3003 jj = 1, ny050
      do 3003 ii = 1, nx050
        if  (rain0(ii,jj).ge.0.0.and.
     #       samp0(ii,jj).ge.1.0.and. 
     #       rain1(ii,jj).ge.0.0)  then
          nn = nn + 1
          s0 = s0 + rain0 (ii,jj)
          s1 = s1 + rain1 (ii,jj)
        end  if
 3003 continue
c
      if  (nn.lt.20)  then
        go  to  3999
      else
        if  (s1.eq.0.0)  then
          fct = 1.0
        else
          fct = s0 / s1
          if  (fct.gt.5.0)  then
            fct = 5.0
          end  if
        end  if
      end  if
c
      fct = 1.0 
c
      do 3004 jj = 1, ny025 
      do 3004 ii = 1, nx025 
        if  (pssm1(ii,jj).gt.0.0)  then
          pssm1 (ii,jj) = fct * pssm1 (ii,jj)
        end  if
 3004 continue
c
 3999 write  (6,*)  '          finished scaling for SSM/I !!', fct  
c
c     ************************************************************************
c     *                                                                      *
c     *  3.  to scale for amsu                                               *
c     *                                                                      *
c     ************************************************************************
c
      fct = -999.0 
      if  (kraw(5).ne.1)  then
        go  to  4999
      end  if
c
      do 4001 jj = 1, ny050
      do 4001 ii = 1, nx050
        rain1 (ii,jj) = pams1 (ii,jj) 
 4001 continue 
c
      nn = 0
      s0 = 0.0
      s1 = 0.0
      do 4003 jj = 1, ny050
      do 4003 ii = 1, nx050
        if  (rain0(ii,jj).ge.0.0.and.
     #       samp0(ii,jj).ge.1.0.and.
     #       rain1(ii,jj).ge.0.0)  then
          nn = nn + 1
          s0 = s0 + rain0 (ii,jj)
          s1 = s1 + rain1 (ii,jj)
        end  if
 4003 continue
c
      if  (nn.lt.20)  then
        go  to  4999
      else
        if  (s1.eq.0.0)  then
          fct = 1.0
        else
          fct = s0 / s1
          if  (fct.gt.5.0)  then
            fct = 5.0
          end  if
        end  if
      end  if
c
      fct = 1.0 
c
      do 4004 jj = 1, ny050 
      do 4004 ii = 1, nx050 
        if  (pams1(ii,jj).gt.0.0)  then
          pams1 (ii,jj) = fct * pams1 (ii,jj)
        end  if
 4004 continue
c
 4999 write  (6,*)  '          finished scaling for AMSU !!', fct  

c 
      return 
      end 
c
c
c
c     ************************************************************************
c     *                                                                      *
c     *                                                                      *
c     *     *******  *******  *******           *******  *     *  *******    *
c     *     *        *           *              *     *  *     *     *       *
c     *     *        *           *              *     *  *     *     *       *
c     *     *  ****  *******     *              *     *  *     *     *       * 
c     *     *  *  *  *           *              *     *  *     *     *       * 
c     *     *     *  *           *              *     *  *     *     *       * 
c     *     *******  *******     *     *******  *******  *******     *       * 
c     *                                                                      *
c     *                                                                      *
c     *                                                                      *
c     *                       *   * ***** *****    ***** ***** ***** *****   *
c     *                       *   * *     *   *    *   * *   * *   * *   *   *
c     *                       *   * ***** *****    ***** ***** *   * *****   *
c     *                        * *  *     *  *  **     *     * *   *     *   *
c     *                         *   ***** *   * ** ***** ***** ***** *****   *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
c     ************************************************************************
c     *                                                                      *
c     *   subroutine:                                                        *
c     *                                                                      *
c     *                get_out                                               *
c     *                                                                      *
c     *   objective :                                                        *
c     *                                                                      *
c     *               to output the pre-processed individual data            *
c     *               into files on hard disks                               *
c     *                                                                      *
c     *   input     :                                                        *
c     *                                                                      *
c     *                                                                      *
c     *                krain1  ==  QCed GTS station observation data         *
c     *                            ngauge integers in total                  *
c     *                                                                      *
c     *                pgts1   ==  gauge-based analysis (mm/day)             *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                sgts1   ==  number of gauges                          *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                egts1   ==  estimated gauge analysis error (mm/day)   *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                dgts1   ==  distance to the closest gauge (km)        *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                pgpi1   ==  QCed GPI precip                           *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                sgpi1   ==  QCed GPI sampling                         *
c     *                            at 0.1 deg grid points                    *
c     *                                                                      *
c     *                pssm1   ==  QCed SSM/I precip                         *
c     *                            at 0.25 deg grid boxes                    *
c     *                                                                      *
c     *                sssm1   ==  QCed SSM/I sampling                       *
c     *                            at 0.25 deg grid boxes                    *
c     *                                                                      *
c     *                pams1   ==  QCed AMSU precp                           *
c     *                            at 0.50 deg grid boxes                    *
c     *                                                                      *
c     *                sams1   ==  QCed AMSU sampling                        *
c     *                            at 0.50 deg grid boxes                    *
c     *                                                                      *
c     *                                                                      *
c     *                                                                      *
c     *  outout    :                                                         *
c     *                                                                      *
c     *                 unit 71  == QCed GTS station data                    *
c     *                             over the target domain                   *
c     *                             krain1 (ngauge)                          *
c     *                                                                      *
c     *                 unit 72  == precipitation analysis at 0.1 deg lat/lon*
c     *                             grid points based on quality-controled   *
c     *                             GTS station observations                 *
c     *                             pgts ( nx000, ny000 ) = precip (mm/day)  *
c     *                             sgts ( nx000, ny000 ) = gauge number     *
c     *                             egts ( nx000, ny000 ) = error  (mm/day)  *
c     *                             dgts ( nx000, ny000 ) = distance (km)    *
c     *                                                                      *
c     *                 unit 73  == QCed GPI estimates                       *
c     *                             at resolution of 0.10 deg grid points    *
c     *                             pgpi1 ( nx000, ny000 ) = precip (mm/day) *
c     *                             sgpi1 ( nx000, ny000 ) = sampling        *
c     *                                                                      *
c     *                 unit 74  == QCed SSM/I Estimates                     *
c     *                             at resolution of 0.25 deg grid boxes     *
c     *                             pssm1 ( nx025, ny025 ) = precip (mm/day) *
c     *                             sssm1 ( nx025, ny025 ) = sampling        *
c     *                                                                      *
c     *                 unit 75  == QCd AMSU Estimates                       *
c     *                             at resolution of 0.50 deg grid boxes     *
c     *                             pams1 ( nx050, ny050 ) = precip (mm/day) *
c     *                             sams1 ( nx050, ny050 ) = sampling        *
c     *                                                                      *
c     *                                                                      *
c     *                                                                      *
c     *                                                                      *
c     ************************************************************************
c
      subroutine   get_out  ( krain1,  
     #                        pgts1, sgts1, egts1, dgts1, 
     #                        pgpi1, sgpi1, 
     #                        pssm1, sssm1, 
     #                        pams1, sams1 ) 
c 
      include  'preproc_v0500_parameter.h' 
c
      integer*2   krain1 (ngauge)
      dimension   pgts1 (nx000,ny000), sgts1 (nx000,ny000),
     #            egts1 (nx000,ny000), dgts1 (nx000,ny000),  
     #            pgpi1 (nx000,ny000), sgpi1 (nx000,ny000),
     #            pssm1 (nx025,ny025), sssm1 (nx025,ny025),
     #            pams1 (nx025,ny025), sams1 (nx025,ny025)
c
c     ************************************************************************
c     *                                                                      *
c     *  1.  to output the QCed GTS station data                             *
c     *                                                                      *
c     ************************************************************************
c
      write  (71,rec=1)    krain1 
      write  (6,*)  '          finished outputing for GTS stn data !!' 
c
c     ************************************************************************
c     *                                                                      *
c     *  2.  to output the gauge-based analysis                              *
c     *                                                                      *
c     ************************************************************************
c
      write  (72,rec=1)    pgts1  
      write  (72,rec=2)    sgts1  
      write  (72,rec=3)    egts1  
      write  (72,rec=4)    dgts1  
      write  (6,*)  '          finished outputing for GTS analysis !!'
c        print *, "PGTS1 is:::::::::::::::::::::",pgts1
c
c     ************************************************************************
c     *                                                                      *
c     *  3.  to output the QCed GPI data                                     *
c     *                                                                      *
c     ************************************************************************
c
      write  (73,rec=1)    pgpi1  
      write  (73,rec=2)    sgpi1  
      write  (6,*)  '          finished outputing for GPI !!'
c
c     ************************************************************************
c     *                                                                      *
c     *  4.  to output the QCed SSM/I data                                   *
c     *                                                                      *
c     ************************************************************************
c
      write  (74,rec=1)    pssm1  
      write  (74,rec=2)    sssm1  
      write  (6,*)  '          finished outputing for SSM/I !!'
c
c     ************************************************************************
c     *                                                                      *
c     *  5.  to output the QCed AMSU data                                    *
c     *                                                                      *
c     ************************************************************************
c
      write  (75,rec=1)    pams1  
      write  (75,rec=2)    sams1  
      write  (6,*)  '          finished outputing for AMSU !!'
c
c
c     ************************************************************************
c     *                                                                      *
c     *  7.  to close the files                                              *
c     *                                                                      *
c     ************************************************************************
c
      close  (unit=71) 
      close  (unit=72) 
      close  (unit=73) 
      close  (unit=74) 
      close  (unit=75) 
      close  (unit=76) 
c
c
      return 
      end 
